from uuid import UUID

import fastapi
from fastapi.security import OAuth2PasswordRequestForm

from .schemas import UserCreateSchema, UserUpdateSchema, UserGetSchema
from .ctrl import UserController
from .services import auth_service, token_service
from ..dependencies import get_user_role


router = fastapi.APIRouter(prefix='/users', tags=['User'])
auth_router = fastapi.APIRouter(prefix='/auth', tags=['Auth'])
ctrl = UserController()


@router.get('')
async def get_users(
    query: UserGetSchema = fastapi.Depends(),
    current_user: str = fastapi.Depends(get_user_role)
):
    return await ctrl.get_list(**query.model_dump(exclude_none=True))


@router.get('/{id}')
async def get_user(
    id: UUID,
    current_user: str = fastapi.Depends(get_user_role)
):
    return await ctrl.get(id)


@router.post('')
async def create_user(
    body: UserCreateSchema,
    current_user: str = fastapi.Depends(get_user_role)
):
    return await ctrl.create(**body.model_dump(exclude_none=True))


@router.patch('/{id}')
async def update_user(
    id: UUID, body: UserUpdateSchema,
    current_user: str = fastapi.Depends(get_user_role)
):
    return await ctrl.update(id, **body.model_dump(exclude_none=True))


@router.delete('/{id}')
async def delete_user(
    id: UUID,
    current_user: str = fastapi.Depends(get_user_role)
):
    return await ctrl.delete(id)


@auth_router.post('/login')
async def login(
    form_data: OAuth2PasswordRequestForm = fastapi.Depends(),
):
    return await auth_service.login(form_data)


@auth_router.get('/users/me')
async def read_user_me(
    user: dict = fastapi.Depends(auth_service.get_current_user)
):
    return UserGetSchema.model_validate(user)


@auth_router.post('/refresh')
async def refresh_token(
    data: dict,
    user: dict = fastapi.Depends(auth_service.get_current_user)
):
    return token_service.refresh_token(data['refresh_token'])
